#include <assert.h>
#include <stdlib.h>
#include "SortedList.h"

/// Returns TRUE if the list is empty.
BOOL isEmpty(SortedList *list)
{
	assert(list != NULL);
	if (list->size == 0)
		return TRUE;
	else
		return FALSE;
}

/// Initialize the list.
void initList(SortedList *list)
{
	assert(list != NULL);

	list->size = 0;
	list->firstNode = NULL;
}

/// Print the contents of the list.
void printList(SortedList *list)
{
	ListNode *node = NULL;
	assert(list != NULL);

	if (isEmpty(list) == TRUE)
		printf("List is empty!\n");

	for (node = list->firstNode; node != NULL; node = node->next) {
		printf(PRINT_MODIFIER" ", node->data);
	}
}

/// Insert data into the list, making sure that the list remains sorted.
void insertData(SortedList *list, Type data)
{
	ListNode *newNode = NULL;
	assert(list != NULL);
	
	newNode = (ListNode *)malloc(sizeof(ListNode));

	// Check for memory allocation errors
	if (newNode == NULL) {
		fprintf(stderr, "Not Enough memory!\nDeleting the list and Exiting!!\n");
		clearList(list);
		exit(1);
	}
	newNode->data = data;
	newNode->next = NULL;

	// If the list is empty, create the first node
	if (isEmpty(list) == TRUE) {
		list->firstNode = newNode;
		list->size++;
	}
	else {
		// Find the position to insert the node.
		ListNode *cur = list->firstNode;
		ListNode *prev = NULL;

		while ((cur != NULL) && (data >= cur->data)) {
			prev = cur;
			cur = cur->next;
		}
		if (prev != NULL) {
			// Insert the node in the list
			newNode->next = cur;
			prev->next = newNode;
		}
		else {
			// We have a new first node
			newNode->next = cur;
			list->firstNode = newNode;
		}
		list->size++;
	}
}

/// Delete the first occurence of data from the list. If deletion was succesfull, return TRUE, else FALSE.
BOOL deleteData(SortedList *list, Type data)
{
	ListNode *cur = NULL;
	ListNode *prev = NULL;

	assert(list != NULL);

	cur = list->firstNode;
	while ((cur != NULL) && (data > cur->data)) {
		prev = cur;
		cur = cur->next;
	}
	if ((cur != NULL) && (cur->data == data)) {
		// Found a node, delete it.
//		ListNode *tmp = node;
		if (cur == list->firstNode) {
			list->firstNode = list->firstNode->next;
			free(cur);
		}
		else {
			prev->next = cur->next;
			free(cur);
		}
		list->size--;
		return TRUE;
	}
	else {
		fprintf(stderr, "Warning: Value "PRINT_MODIFIER" was not found in the list!\n", data);
		return FALSE;
	}
}

/// Access the data at position index. Return NULL if unsuccesfull.
ListNode *accessNode(SortedList *list, int index)
{
	ListNode *node = NULL;

	assert(list != NULL);

	if (index >= list->size)
		return NULL;
	else {
		int i = 0;
		node = list->firstNode;
		for (i = 0; i < index; i++) {
			node = node->next;
		}
		return node;
	}
}

/// Delete the minimum valued node from the list. Return FALSE if the list was empty before calling.
BOOL deleteMin(SortedList *list)
{
	ListNode *node = NULL;
	assert(list != NULL);

	// Deletes the first node in the list
	if (isEmpty(list) == FALSE) {
		node = list->firstNode;
		list->firstNode = node->next;
		list->size--;

		free(node);
		return TRUE;
	}
	return FALSE;
}

/// Delete the minimum valued node from the list. Return FALSE if the list was empty before calling.
BOOL deleteMax(SortedList *list)
{
	ListNode *node = NULL;
	ListNode *prev = NULL;
	assert(list != NULL);

	if (isEmpty(list) == FALSE) {
		node = list->firstNode;
		prev = NULL;
		while(node->next != NULL) {
			prev = node;
			node = node->next;
		}
		if (prev == NULL) {
			// This is the first node
			list->firstNode = NULL;
		}
		else {
			prev->next = NULL;
		}

		list->size--;

		free(node);
		return TRUE;
	}

	return FALSE;
}

/// Clear the contents of the list.
void clearList(SortedList *list)
{
	ListNode *next = NULL;
	ListNode *cur = NULL;

	assert(list != NULL);

	fprintf(stdout, "Freeing memory\n");
	if (isEmpty(list) == FALSE) {
		cur = list->firstNode;
		next = cur->next;
		while (next != NULL) {
			next = cur->next;
			free(cur);
			cur = next;
		}
	}	
}

/// Find the first node with key value key.
ListNode *findNode(SortedList *list, Type key)
{
	ListNode *cur = NULL;

	assert(list != NULL);

	cur = list->firstNode;
	while ((cur != NULL) && (key > cur->data)) {
		cur = cur->next;
	}
	if ((cur != NULL) && (cur->data == key)) {
		// Found a node, return it.
		return cur;
	}
	else {
		fprintf(stderr, "Warning: Value "PRINT_MODIFIER" was not found in the list!\n", key);
		return FALSE;
	}
}

/// Check if a value is in the list, NULL if non existant.
BOOL exists(SortedList *list, Type key)
{
	if (findNode(list, key) != NULL)
		return TRUE;
	else
		return FALSE;
}
