#ifndef __SORTEDLIST_H__
#define __SORTEDLIST_H__

#include <stdio.h>
#include "defns.h"

/**
 * A list node.
 * Contains the data and a pointer to the next node.
 */
typedef struct _ListNode {
	Type data;					// Our data.
	struct _ListNode *next;		// A pointer to the next element in the list. NULL if this is the last node.
} ListNode;

/**
 * Typedef a new name for a pointer to a ListNode.
 */
typedef ListNode * ListNodePtr;


/**
 * A Sorted List.
 * Contains the size of the list and a pointer to the first node. If this is NULL, then the list is empty.
 */
typedef struct _SortedList {
	int size;				// The size of the list.
	ListNode *firstNode;	// A pointer to the first node in the list.
} SortedList;

/**
 * Typedef a new name for a pointer to a SortedList.
 */
typedef SortedList * SortedListPtr;

/**
 * Forward Function Declarations.
 */

/// Returns TRUE if the list is empty.
BOOL isEmpty(SortedList *list);

/// Initialize the list.
void initList(SortedList *list);

/// Print the contents of the list.
void printList(SortedList *list);

/// Insert data into the list, making sure that the list remains sorted.
void insertData(SortedList *list, Type data);

/// Delete the first occuerence of data from the list. If deletion was succesfull, return TRUE, else FALSE.
BOOL deleteData(SortedList *list, Type data);

/// Access the data at position index. Return NULL if unsuccesfull.
ListNode *accessNode(SortedList *list, int index);

/// Delete the minimum valued node from the list. Return FALSE if the list was empty before calling.
BOOL deleteMin(SortedList *list);

/// Delete the minimum valued node from the list. Return FALSE if the list was empty before calling.
BOOL deleteMax(SortedList *list);

/// Clear the contents of the list.
void clearList(SortedList *list);

/// Find the first occurence of the list node with value key, NULL if non existant.
ListNode *findNode(SortedList *list, Type key);

/// Check if a value is in the list, NULL if non existant.
BOOL exists(SortedList *list, Type key);

#endif // __SORTEDLIST_H__